/* FORMANT1 - sample code for a basic filter (band-pass) instrument

   p0 = output start time
   p1 = input start time
   p2 = input duration
   p3 = amplitude multiplier
   p4 = filter center frequency (Hz)
   p5 = filter bandwidth (Hz)
   p6 = input channel [optional, default is 0]
   p7 = pan (in percent-to-left format) [optional, default is 0]
*/

#include <stdio.h>
#include <stdlib.h>
#include <ugens.h>
#include <Ougens.h>
#include "FORMANT1.h"          // declarations for this instrument class
#include <rt.h>
#include <rtdefs.h>


FORMANT1::FORMANT1()
	: resetter(0)
{
}

FORMANT1::~FORMANT1()
{
}


int FORMANT1::init(double p[], int n_args)
{
    if (rtsetoutput(p[0], p[2], this) == -1)
		return DONT_SCHEDULE;

	if (outputChannels() > 2)
		return die("FORMANT1", "Use mono or stereo output only.");

	if (rtsetinput(p[1], this) == -1)
		return DONT_SCHEDULE;

    inchan = p[6];
	if (inchan >= inputChannels())
		return die("FORMANT1", "You asked for channel %d of a %d-channel input.",
		                                             inchan, inputChannels());

    // set up our input object
    theInput = new Ortgetin(this);

    // set up our filter
    // Oreson(float SR, float centerFreq, float bandwidth[, Scale scaling])
    theFilt = new Oreson(SR, p[4], p[5]);
    
    amp = p[3];
    pan = p[7];

	return nSamps();
}


int FORMANT1::configure()
{
    return 0;	// IMPORTANT: Return 0 on success, and -1 on failure.
}


void FORMANT1::doupdate()
{
	double p[8];
	update(p, 8);

	amp = p[3];
    pan = p[7];
}


int FORMANT1::run()
{
    float out[2];
    float in[2];

	// Each loop iteration processes 1 sample frame.
	for (int i = 0; i < framesToRun(); i++) {
		if (--resetter <= 0) {
			doupdate();
			resetter = getSkip();
		}

        // Grab the current input sample, scaled by the amplitude multiplier.
        theInput->next(in);

        // filter it!
		out[0] = theFilt->next(in[inchan]) * amp;

		// If we have stereo output, use the pan pfield.
		if (outputChannels() == 2) {
			out[1] = out[0] * (1.0 - pan);
			out[0] *= pan;
		}

		// Write this sample frame to the output buffer.
		rtaddout(out);

		// Increment the count of sample frames this instrument has written.
		increment();
	}

	// Return the number of frames we processed.
	return framesToRun();
}


Instrument *makeFORMANT1()
{
	FORMANT1 *inst = new FORMANT1();
	inst->set_bus_config("FORMANT1");

	return inst;
}


void rtprofile()
{
	RT_INTRO("FORMANT1", makeFORMANT1);
}


